<?php

class Magento
{
  public static function validatePassword($hash, $password, $salt)
  {
    $argonPasswordsArr = self::getArgonPasswordMage(
      $password, $salt, $hash
    );

    $possibleSalt = explode(':', $hash);
    $possibleSalt = isset($possibleSalt[1]) ? $possibleSalt[1] : '';

    if ($possibleSalt) {
      $argonPasswordsArr = array_merge(
        $argonPasswordsArr, self::getArgonPasswordMage(
        $password, $possibleSalt, $hash
      )
      );
    }

    return (md5($salt . $password) === $hash
      || md5($salt . $password) . ':' . $salt === $hash
      || hash('sha256', $salt . $password) === $hash
      || hash('sha256', $salt . $password) . ':' . $salt === $hash
      || in_array($hash, $argonPasswordsArr, true)
      || in_array($hash . ':' . $salt, $argonPasswordsArr, true)
      || ($possibleSalt && in_array($hash . ':' . $possibleSalt, $argonPasswordsArr, true))
    );
  }

  public static function getArgonPasswordMage($password, $salt, $currentHash)
  {
    $result = [];

    $hash = '';
    $version = explode(':', $currentHash);
    $version = isset($version[2]) ? $version[2] : '';

    $argon2Id13 = 2;
    $argon2Id13Agnostic = 3;
    $seedBytes = 32;
    $opsLimit = 2;
    $memLimit = 67108864;
    $saltBytes = 16;
    $hashAlgo = 2;

    if (version_compare(PHP_VERSION, '7.0.0', '>=') && function_exists('sodium_crypto_pwhash')) {
      defined('SODIUM_CRYPTO_SIGN_SEEDBYTES') && $seedBytes = SODIUM_CRYPTO_SIGN_SEEDBYTES;
      defined('SODIUM_CRYPTO_PWHASH_OPSLIMIT_INTERACTIVE') && $opsLimit = SODIUM_CRYPTO_PWHASH_OPSLIMIT_INTERACTIVE;
      defined('SODIUM_CRYPTO_PWHASH_MEMLIMIT_INTERACTIVE') && $memLimit = SODIUM_CRYPTO_PWHASH_MEMLIMIT_INTERACTIVE;
      defined('SODIUM_CRYPTO_PWHASH_SALTBYTES') && $saltBytes = SODIUM_CRYPTO_PWHASH_SALTBYTES;
      defined('SODIUM_CRYPTO_PWHASH_ALG_ARGON2ID13') && $hashAlgo = SODIUM_CRYPTO_PWHASH_ALG_ARGON2ID13;

      $argon2Id13Data = explode('_', $version);
      $argon2Id13DataAlgorithm = isset($argon2Id13Data[0]) ? $argon2Id13Data[0] : '';

      switch ($argon2Id13DataAlgorithm) {
        case $argon2Id13Agnostic:
          $hash = self::getArgonHashMage(
            $password,
            isset($argon2Id13Data[1]) ? $argon2Id13Data[1] : $seedBytes,
            isset($argon2Id13Data[2]) ? $argon2Id13Data[2] : $opsLimit,
            isset($argon2Id13Data[3]) ? $argon2Id13Data[3] : $memLimit,
            $salt,
            $saltBytes,
            $hashAlgo
          );
          break;
        case $argon2Id13:
        default:
          $hash = self::getArgonHashMage($password, $seedBytes, $opsLimit, $memLimit, $salt, $saltBytes, $hashAlgo);
          break;
      }

      if ($hash !== '') {
        $result = [
          implode(':', [$hash, $salt, $version]),
          implode(':', [$hash, $salt, $argon2Id13]),
          implode(':', [$hash, $salt, $argon2Id13Agnostic]),
          implode(':', [$hash, $salt, $argon2Id13Agnostic . '_' . $seedBytes . '_' . $opsLimit . '_' . $memLimit]),
          $hash . ':' . $salt
        ];
      }
    }

    return $result;
  }

  public static function getArgonHashMage($password, $seedBytes, $opsLimit, $memLimit, $salt, $saltBytes, $hashAlgo) {
    if (empty($salt)) {
      return $password;
    }

    if (strlen($salt) < $saltBytes) {
      $salt = str_pad($salt, $saltBytes, $salt);
    } elseif (strlen($salt) > $saltBytes) {
      $salt = substr($salt, 0, $saltBytes);
    }

    return bin2hex(sodium_crypto_pwhash($seedBytes, $password, $salt, $opsLimit, $memLimit, $hashAlgo));
  }
}